@@ -491,7 +491,7 @@ class Key(object):
 
 		#
 
-# the sshfs object class.
+# the ssh object class.
 class SSH(object):
 	def __init__(self):
 
@@ -527,7 +527,7 @@ class SSH(object):
 		#
 	def command(self, 
 		# the function parameters.
-		command=["ls"], 
+		command="ls", 
 		# the return format: string / array
 		return_format="string",
 		# the ssh parameters.
@@ -549,12 +549,24 @@ class SSH(object):
 			key_path=key_path,)
 		
 		# execute.
-		command = base
+		if isinstance(command, list):
+			s = ""
+			for i in command: 
+				if s == "": s = i
+				else: s += " "+i
+			command = base + [s]
+		else:
+			command = base + [command]
 
 		# do.
+		s = ""
+		for i in command: 
+			if s == "": s = i
+			else: s += " "+i
+		#print("COMMAND:",s)
 		output = utils.__execute__(command, shell=False, return_format=return_format)
+		#print("OUTPUT:",output)
 		#output = utils.__execute_script__(utils.__array_to_string__(command, joiner="\n"), shell=False, return_format=return_format)
-		
 
 		response = utils.__default_response__()
 		
@@ -568,6 +580,9 @@ class SSH(object):
 			response["error"] = f"Error: {error}."
 			return response
 
+		elif " Permission denied (publickey)." in output:
+			return r3sponse.error_response("Permission denied (publickey).")
+
 		# unkown host.
 		elif "Could not resolve hostname " in output:
 			hostname = output.split('Could not resolve hostname ')[1].split(":")[0]
@@ -873,7 +888,7 @@ class Agent(object):
 		# variables.
 		#self.initialize()
 		
-		# set agent.
+		"""# set agent.
 		self.SSH_AUTH_SOCK = os.environ.get("SSH_AUTH_SOCK")
 		self.SSH_AGENT_PID = os.environ.get("SSH_AGENT_PID")
 		if self.SSH_AUTH_SOCK == None or self.SSH_AGENT_PID == None:
@@ -901,6 +916,39 @@ class Agent(object):
 				os.environ["SSH_AGENT_PID"] = SSH_AGENT_PID
 			except: a=1
 		except: a=1
+		"""
+
+		# set agent.
+		self.SSH_AUTH_SOCK = os.environ.get("SSH_AUTH_SOCK")
+		self.SSH_AGENT_PID = os.environ.get("SSH_AGENT_PID")
+		if self.SSH_AUTH_SOCK == None or self.SSH_AGENT_PID == None:
+			utils.__execute_script__("pkill -9 -f ssh-agent")
+			output = subprocess.check_output(["ssh-agent"]).decode()
+			self.SSH_AUTH_SOCK = output.split("SSH_AUTH_SOCK=")[1].split(";")[0]
+			self.SSH_AGENT_PID = output.split("SSH_AGENT_PID=")[1].split(";")[0]
+			os.environ["SSH_AUTH_SOCK"] = self.SSH_AUTH_SOCK
+			os.environ["SSH_AGENT_PID"] = self.SSH_AGENT_PID
+			utils.__execute_script__("ssh-add -D")
+
+		# set agent nr.2.
+		#try:
+		#	output = utils.__execute__([f"ssh-add", "-D"])
+		#except: a=1
+		#try:
+		#	output = utils.__execute__([f"ssh-add", "-k"])
+		#except: a=1
+		#try:
+		#	output = utils.__execute__(f"ssh-agent")
+		#	try: 
+		#		SSH_AUTH_SOCK = output.split("SSH_AUTH_SOCK=")[1].split(";")[0]
+		#		os.environ["SSH_AUTH_SOCK"] = SSH_AUTH_SOCK
+		#	except: a=1
+		#	try: 
+		#		SSH_AGENT_PID = output.split("SSH_AGENT_PID=")[1].split(";")[0]
+		#		os.environ["SSH_AGENT_PID"] = SSH_AGENT_PID
+		#	except: a=1
+		#except: a=1
+
 		
 		#
 	def add(self, 
@@ -1261,6 +1309,9 @@ class SSHD(object):
 				# allowed connection options.
 				"x11_forwarding":False,
 				"tcp_forwarding":False,
+				"permit_tunnel":False,
+				"allow_stream_local_forwarding":False,
+				"gateway_ports":False,
 			},
 		},
 	):
@@ -1352,9 +1403,9 @@ class SSHD(object):
 			configuration += f'\n    AllowTcpForwarding {self.__convert_boolean__(info["tcp_forwarding"])}'
 
 			# default options.
-			configuration += f'\n    PermitTunnel no'
-			configuration += f'\n    AllowStreamLocalForwarding no'
-			configuration += f'\n    GatewayPorts no'
+			configuration += f'\n    PermitTunnel {self.__convert_boolean__(info["permit_tunnel"])}'
+			configuration += f'\n    AllowStreamLocalForwarding {self.__convert_boolean__(info["allow_stream_local_forwarding"])}'
+			configuration += f'\n    GatewayPorts {self.__convert_boolean__(info["gateway_ports"])}'
 			configuration += f'\n    PermitTTY yes'
 			
 			# check ip filter.
@@ -1551,6 +1602,27 @@ class SSHD(object):
 		#
 	#
 
+# the ssh connections object class.
+class Connections(object):
+	def __init__(self):
+		a=1
+	def list(self, filter="ssh"):
+		output = syst3m.utils.__execute_script__("""ss | grep ssh | awk '{print $1","$2","$3","$4","$5","$6}' """)
+		connections = {}
+		for line in output.split("\n"):
+			if line not in [""]:
+				net_id,state,recvq, sendq,local_address,remote_address = line.split(",")
+				if state == "ESTAB":
+					connections[remote_address] = {
+						"remote_address":remote_address,
+						"local_address":local_address,
+						"recvq":recvq,
+						"sendq":sendq,
+						"net_id":net_id,
+					}
+		return r3sponse.success_response(f"Successfully listed {len(connections)} ssh connection(s).", {
+			connections:connections,
+		})
 # Initialized classes.
 installation = Installation()
 config = Config()
@@ -1560,6 +1632,7 @@ sshfs = SSHFS()
 scp = SCP()
 agent = Agent()
 key = Key()
+connections = Connections()
 
 """
 
